//-----------------------------------------------------------------------------
#include <apps/Common/wxAbstraction.h>
#include "CaptureThread.h"
#include "PropViewFrame.h"
#include <wx/event.h>

//-----------------------------------------------------------------------------
class TimerLite
//-----------------------------------------------------------------------------
{
	clock_t m_endClocks; 
	clock_t Delay2Clocks( unsigned long nextDelay_ms )
	{
		return ( nextDelay_ms * CLOCKS_PER_SEC ) / 1000;
	}
public:
	TimerLite() : m_endClocks(0) {;}
	TimerLite( unsigned long nextDelay_ms )
	{
		Restart( nextDelay_ms );
	}
	void Restart( unsigned long nextDelay_ms )
	{
		m_endClocks = clock() + Delay2Clocks( nextDelay_ms );
	}
	bool IsExpired( unsigned long nextDelay_ms )
	{
		clock_t curClocks = clock();
		if( curClocks > m_endClocks )
		{
			m_endClocks = curClocks + Delay2Clocks( nextDelay_ms );
			return true;
		}
		return false;
	}
};

using namespace std;
using namespace mvIMPACT::acquire;

unsigned int CaptureThread::m_activeInstance = 0;
unsigned int CaptureThread::m_instanceCnt = 0;

//-----------------------------------------------------------------------------
CaptureThread::CaptureThread( FunctionInterface* pFuncInterface, Device* pDev, wxWindow* pParentWindow ) :
	wxThread(wxTHREAD_JOINABLE), m_boLive(false), m_boRecord(false), m_boContinuousRecording(false), m_boForwardIncompleteFrames(false),
	m_currentRequest(INVALID_ID), m_discardedImages(0), m_instanceNr(m_instanceCnt++), m_nextPendingRequest(INVALID_ID),
	m_parentWindowID(pParentWindow->GetId()), m_pDev(pDev), m_pFuncInterface(pFuncInterface),
	m_ImageRequestControl(pDev), m_pParentWindowEventHandler(pParentWindow->GetEventHandler()),
	m_currentSequenceIndexToDisplay(0), m_currentCountToRecord(0), m_totalCountToRecord(0),
	m_captureQueueDepth(0), m_currentlyActiveRequests(0)
//-----------------------------------------------------------------------------
{

}

//-----------------------------------------------------------------------------
CaptureThread::~CaptureThread()
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	if( m_nextPendingRequest != INVALID_ID )
	{
		m_pFuncInterface->imageRequestUnlock( m_nextPendingRequest );
		m_nextPendingRequest = INVALID_ID;
	}

	if( m_currentRequest != INVALID_ID )
	{
		m_pFuncInterface->imageRequestUnlock( m_currentRequest );
		m_currentRequest = INVALID_ID;
	}
}

//-----------------------------------------------------------------------------
TRequestResult CaptureThread::GetImageData( RequestData& requestData, bool boFillInfoVector, std::vector<wxString>& infoStrings ) const
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	if( m_nextPendingRequest != INVALID_ID )
	{
		if( m_currentRequest != INVALID_ID )
		{
			UnlockRequest( m_currentRequest );
		}
		m_currentRequest = m_nextPendingRequest;
		m_nextPendingRequest = INVALID_ID;
	}

	if( m_currentRequest != INVALID_ID )
	{
		const Request* pReq = m_pFuncInterface->getRequest( m_currentRequest );
		if( pReq )
		{
			ImageBufferDesc nbd(pReq->getImageBufferDesc());
			requestData.image_ = nbd;
			//requestData.image_ = ImageBufferDesc(pReq->getImageBufferDesc()); // doesn't work with GCC...
			requestData.requestInfo_.cameraOutputUsed = ( pReq->infoCameraOutputUsed.isValid() ) ? pReq->infoCameraOutputUsed.read() : coUndefined;
			requestData.requestInfo_.exposeStart_us = pReq->infoExposeStart_us.read();
			requestData.requestInfo_.exposeTime_us = pReq->infoExposeTime_us.read();
			requestData.requestInfo_.frameID = pReq->infoFrameID.read();
			requestData.requestInfo_.frameNr = pReq->infoFrameNr.read();
			requestData.requestInfo_.gain_dB = pReq->infoGain_dB.read();
			requestData.requestInfo_.lineCounter = ( pReq->infoLineCounter.isValid() ) ? pReq->infoLineCounter.read() : -1;
			requestData.requestInfo_.timeStamp_us = pReq->infoTimeStamp_us.read();
			requestData.requestInfo_.transferDelay_us = ( pReq->infoTransferDelay_us.isValid() ) ? pReq->infoTransferDelay_us.read() : -1;
			requestData.requestInfo_.videoChannel = (pReq->infoVideoChannel.isValid() ) ? pReq->infoVideoChannel.read() : -1;
			requestData.bayerParity_ = pReq->imageBayerMosaicParity.read();
			if( requestData.bayerParity_ != bmpUndefined )
			{
				requestData.pixelFormat_ = wxString::Format( wxT("%s(Bayer pattern: %s)"), ConvertedString(pReq->imagePixelFormat.readS()).c_str(), ConvertedString(pReq->imageBayerMosaicParity.readS()).c_str() );
			}
			else
			{
				requestData.pixelFormat_ = ConvertedString(pReq->imagePixelFormat.readS());
			}
			if( boFillInfoVector )
			{
				ComponentIterator it(pReq->getInfoIterator());
				while( it.isValid() )
				{
					if( it.isProp() && it.isVisible() )
					{
						Property prop(it);
						infoStrings.push_back( wxString::Format( wxT("%s: %s"), ConvertedString(prop.name()).c_str(), ConvertedString(prop.readS()).c_str() ) );
					}
					++it;
				}
			}
			return pReq->requestResult.read();
		}
	}
	else
	{
		requestData = RequestData();
		return rrOK;
	}
	return rrError;
}

//-----------------------------------------------------------------------------
size_t CaptureThread::DisplaySequenceRequest( size_t nr )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	m_currentSequenceIndexToDisplay = ( nr < m_recordedSequence.size() ) ? nr : m_recordedSequence.size() - 1;
	int req = m_recordedSequence[m_currentSequenceIndexToDisplay];
	if( req != m_currentRequest )
	{
		if( m_nextPendingRequest != INVALID_ID )
		{
			UnlockRequest( m_nextPendingRequest );
		}
		m_nextPendingRequest = req;
	}

	return m_currentSequenceIndexToDisplay;
}

//-----------------------------------------------------------------------------
size_t CaptureThread::DisplaySequenceRequestNext( void )
//-----------------------------------------------------------------------------
{
	return DisplaySequenceRequest( m_currentSequenceIndexToDisplay + 1 );
}

//-----------------------------------------------------------------------------
size_t CaptureThread::DisplaySequenceRequestPrev( void )
//-----------------------------------------------------------------------------
{
	return DisplaySequenceRequest( ( m_currentSequenceIndexToDisplay != 0 ) ? ( m_currentSequenceIndexToDisplay - 1 ) : 0 );
}

//-----------------------------------------------------------------------------
void* CaptureThread::Entry( void )
//-----------------------------------------------------------------------------
{
	const unsigned long waitBeforeBreak = 60;
	TimerLite forceBreakTimer( waitBeforeBreak );
	while( !TestDestroy() )
	{
		int requestNr;
		if( m_boLive )
		{
			RequestImages();
		}
		if( ( requestNr = m_pFuncInterface->imageRequestWaitFor( 0, 0 ) ) < 0 )
		{
			if( ( requestNr = m_pFuncInterface->imageRequestWaitFor( 200, 0 ) ) >= 0 )
			{
				forceBreakTimer.Restart( waitBeforeBreak );
			}
		}
		else if( forceBreakTimer.IsExpired( waitBeforeBreak ) )
		{
			Sleep( 10 );
		}
		if( requestNr >= 0 )
		{
			wxCriticalSectionLocker locker(m_critSect);
			if( m_pFuncInterface->isRequestNrValid( requestNr ) )
			{
				const Request* pReq = m_pFuncInterface->getRequest( requestNr );
				if( pReq )
				{
					if( m_activeInstance == m_instanceNr )
					{
						{
							wxCriticalSectionLocker locker(m_critSectConfig);
							--m_currentlyActiveRequests;
							m_lastRequestResult = ConvertedString(pReq->requestResult.readS().c_str());
						}
						TRequestResult rr = pReq->requestResult.read();
						if( ( rr == rrOK ) || ( ( rr == rrFrameIncomplete ) && m_boForwardIncompleteFrames ) )
						{
							RequestInfoData* pReqInfo = new RequestInfoData();
							pReqInfo->cameraOutputUsed = ( pReq->infoCameraOutputUsed.isValid() ) ? pReq->infoCameraOutputUsed.read() : coUndefined;
							pReqInfo->exposeStart_us = pReq->infoExposeStart_us.read();
							pReqInfo->exposeTime_us = pReq->infoExposeTime_us.read();
							pReqInfo->frameID = pReq->infoFrameID.read();
							pReqInfo->frameNr = pReq->infoFrameNr.read();
							pReqInfo->gain_dB = pReq->infoGain_dB.read();
							pReqInfo->lineCounter = ( pReq->infoLineCounter.isValid() ) ? pReq->infoLineCounter.read() : -1;
							pReqInfo->timeStamp_us = pReq->infoTimeStamp_us.read();
							pReqInfo->transferDelay_us = ( pReq->infoTransferDelay_us.isValid() ) ? pReq->infoTransferDelay_us.read() : -1;
							pReqInfo->videoChannel = (pReq->infoVideoChannel.isValid() ) ? pReq->infoVideoChannel.read() : -1;
							wxCommandEvent eInfo(imageInfoEvent, m_parentWindowID);
							eInfo.SetClientData( pReqInfo );
							wxPostEvent( m_pParentWindowEventHandler, eInfo );
							if( m_boRecord )
							{
								m_recordedSequence.push_back( requestNr );
							}
							if( m_nextPendingRequest != INVALID_ID )
							{
								UnlockRequest( m_nextPendingRequest );
								++m_discardedImages;
								if( m_SkippedImageCount++ == 0 )
								{
									wxCommandEvent e( imageSkippedEvent, m_parentWindowID );
									wxPostEvent( m_pParentWindowEventHandler, e );
								}
								m_nextPendingRequest = requestNr;
							}
							else
							{
								m_nextPendingRequest = requestNr;
								wxCommandEvent e(imageReadyEvent, m_parentWindowID);
								wxPostEvent( m_pParentWindowEventHandler, e );
							}
						}
						else
						{
							if( rr & rrUnprocessibleRequest )
							{
								m_lastErrorMessage = ConvertedString(pReq->requestResult.readS());
							}
							m_pFuncInterface->imageRequestUnlock( requestNr );
							if( GetLiveMode() )
							{
								// Livemode is stopped depending on requestresult, TRequestResult
								if( rr & rrUnprocessibleRequest )
								{
									InternalSetLiveMode( false );
									m_pFuncInterface->imageRequestReset( 0, 0 );
									wxCommandEvent e(liveModeAborted, m_parentWindowID);
									e.SetString(m_lastErrorMessage);
									wxPostEvent( m_pParentWindowEventHandler, e );
									if( m_currentCountToRecord )
									{
										m_currentCountToRecord = 1;
									}
								}
							}
						}

						if( m_boContinuousRecording )
						{
							const int requestCount = m_pFuncInterface->requestCount();
							{
								wxCriticalSectionLocker locker(m_critSectConfig);
								RequestContainer::size_type maxSequenceSize = static_cast<RequestContainer::size_type>(( m_captureQueueDepth > 0 ) ? requestCount - m_captureQueueDepth : requestCount / 2);
								if( m_recordedSequence.size() > maxSequenceSize )
								{
									FreeLastRequestFromSequence( m_nextPendingRequest );
								}
							}
						}
						else if( m_currentCountToRecord && m_boRecord )
						{
							--m_currentCountToRecord;
							if( ( m_currentCountToRecord == 0 ) || ( m_recordedSequence.size() == m_pFuncInterface->requestCount() ) )
							{
								InternalSetLiveMode( false );
								wxCommandEvent e( sequenceReadyEvent, m_parentWindowID );
								wxPostEvent( m_pParentWindowEventHandler, e );
							}
						}
					}
					else
					{
						m_pFuncInterface->imageRequestUnlock( requestNr );
						++m_discardedImages;
					}
				}
			}
		}
	}
	return 0;
}

//-----------------------------------------------------------------------------
int CaptureThread::ExchangeLastSnapResult( int newResult ) const
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	int oldval = m_currentRequest;
	m_currentRequest = newResult;
	return oldval;
}

//-----------------------------------------------------------------------------
wxString CaptureThread::GetLastRequestResult( void ) const
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSectConfig);
	return m_lastRequestResult;
}

//-----------------------------------------------------------------------------
int CaptureThread::GetLastSnapResult( void ) const
//-----------------------------------------------------------------------------
{
	return ExchangeLastSnapResult( INVALID_ID );
}

//-----------------------------------------------------------------------------
void CaptureThread::SetActive( void )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	m_activeInstance = m_instanceNr;
}

//-----------------------------------------------------------------------------
void CaptureThread::SetCaptureMode( bool boForwardIncompleteFrames )
//-----------------------------------------------------------------------------
{
	m_boForwardIncompleteFrames = boForwardIncompleteFrames;
}

//-----------------------------------------------------------------------------
void CaptureThread::SetContinuousRecording( bool boContinuous )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	m_boContinuousRecording = boContinuous;
}

//-----------------------------------------------------------------------------
void CaptureThread::SetCaptureQueueDepth( int queueDepth )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSectConfig);
	m_captureQueueDepth = queueDepth;
}

//-----------------------------------------------------------------------------
void CaptureThread::UnlockRequest( int nr ) const
//-----------------------------------------------------------------------------
{
	// unlock of request only when not in sequence
	// ( a request usage counter would be better )
	RequestContainer::const_iterator it = m_recordedSequence.begin();
	RequestContainer::const_iterator itEND = m_recordedSequence.end();
	while( it != itEND )
	{
		if( *it == nr )
		{
			break;
		}
		++it;
	}
	if( it == itEND )
	{
		m_pFuncInterface->imageRequestUnlock( nr );
	}
}

//-----------------------------------------------------------------------------
void CaptureThread::FreeSequence( void )
//-----------------------------------------------------------------------------
{
	while( !m_recordedSequence.empty() )
	{
		FreeLastRequestFromSequence( m_currentRequest );
	}
	m_currentSequenceIndexToDisplay = 0;
}

//-----------------------------------------------------------------------------
void CaptureThread::FreeLastRequestFromSequence( int& requestNotToUnlock )
//-----------------------------------------------------------------------------
{
	int nr = m_recordedSequence.front();
	if( nr != requestNotToUnlock )
	{
		m_pFuncInterface->imageRequestUnlock( nr );
	}
	m_recordedSequence.pop_front();
}

//-----------------------------------------------------------------------------
int CaptureThread::RequestSingle( bool boInformUserOnRecordStart )
//-----------------------------------------------------------------------------
{
	if( !m_boRecord )
	{
		FreeSequence();
	}

	int result = m_pFuncInterface->imageRequestSingle( &m_ImageRequestControl );
	if( m_boRecord && ( result < 0 ) && CheckSequenceRestart( boInformUserOnRecordStart ) )
	{
		SetRecordMode( true );
		result = RequestSingle( boInformUserOnRecordStart );
	}

	return result;
}

//-----------------------------------------------------------------------------
int CaptureThread::InternalSetRecordMode( bool boOn )
//-----------------------------------------------------------------------------
{
	if( boOn )
	{
		m_currentSequenceIndexToDisplay = 0;
		m_pFuncInterface->imageRequestReset( 0, 0 );
		if( m_nextPendingRequest != INVALID_ID )
		{
			m_pFuncInterface->imageRequestUnlock( m_nextPendingRequest );
			m_nextPendingRequest = INVALID_ID;
		}
		FreeSequence();
		m_currentCountToRecord = ( m_totalCountToRecord == 0 ) ? m_pFuncInterface->requestCount() : m_totalCountToRecord;
	}
	m_boRecord = boOn;

	return m_boRecord;
}

//-----------------------------------------------------------------------------
int CaptureThread::SetRecordMode( bool boOn )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	return InternalSetRecordMode( boOn );
}

//-----------------------------------------------------------------------------
void CaptureThread::SetRecordSequenceSize( size_t sequenceSize )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	m_totalCountToRecord = sequenceSize;
}

//-----------------------------------------------------------------------------
bool CaptureThread::CheckSequenceRestart( bool boInformUserOnRecordStart ) const
//-----------------------------------------------------------------------------
{
	if( !boInformUserOnRecordStart )
	{
		return true;
	}

	bool result = false;
	wxMessageDialog dlg( NULL,
		wxString::Format( wxT("Restarts the image recording. All current recorded images (%lu) will be lost.\nCheck 'Settings -> Silent recording' to get rid of this and other questions."), static_cast<unsigned long int>(m_recordedSequence.size()) ),
		wxT("Warning"),
		wxNO_DEFAULT | wxYES_NO | wxICON_INFORMATION );

	if( dlg.ShowModal() == wxID_YES )
	{
		result = true;
	}

	return result;
}

//-----------------------------------------------------------------------------
int CaptureThread::InternalSetLiveMode( bool boOn, bool boInformUserOnRecordStart /* = true */ )
//-----------------------------------------------------------------------------
{
	if( boOn )
	{
		m_pFuncInterface->imageRequestReset( 0, 0 );
		if( m_nextPendingRequest != INVALID_ID )
		{
			m_pFuncInterface->imageRequestUnlock( m_nextPendingRequest );
			m_nextPendingRequest = INVALID_ID;
		}

		if( m_boRecord )
		{
			if( ( m_recordedSequence.size() != 0 ) && CheckSequenceRestart( boInformUserOnRecordStart ) )
			{
				InternalSetRecordMode( true );
			}
		}
		else
		{
			FreeSequence();
		}

		m_discardedImages = 0;
		m_SkippedImageCount = 0;
		int framesRequested = 0;
		int result = RequestImages( &framesRequested );
		if( ( result == DMR_NO_ERROR ) ||
			( ( result == DEV_NO_FREE_REQUEST_AVAILABLE ) && ( framesRequested > 0 ) ) )
		{
			m_boLive = boOn;
		}
		else if( result != DEV_NO_FREE_REQUEST_AVAILABLE )
		{
			wxMessageDialog dlg( NULL, wxString::Format( wxT("Live not started(Reason: %s)! More information can be found in the *.log-file or the debug output.\n"), ConvertedString(ImpactAcquireException::getErrorCodeAsString( result )).c_str() ), wxT("Error"), wxOK | wxICON_WARNING );
			dlg.ShowModal();
		}
	}
	else
	{
		m_boLive = boOn;
	}

	return m_boLive;
}

//-----------------------------------------------------------------------------
int CaptureThread::SetLiveMode( bool boOn, bool boInformUserOnRecordStart /* = true */ )
//-----------------------------------------------------------------------------
{
	wxCriticalSectionLocker locker(m_critSect);
	return InternalSetLiveMode( boOn, boInformUserOnRecordStart );
}

//-----------------------------------------------------------------------------
int CaptureThread::RequestImages( int* pFramesRequested /* = 0 */ )
//-----------------------------------------------------------------------------
{
	int requestCnt = 0;
	int requestResult = DMR_NO_ERROR;
	do
	{
		{
			wxCriticalSectionLocker locker(m_critSectConfig);
			if( m_boRecord && ( m_totalCountToRecord > 0 ) && ( m_currentlyActiveRequests - m_currentCountToRecord <= 0 ) )
			{
				break;
			}
		}
		if( ( requestResult = m_pFuncInterface->imageRequestSingle( &m_ImageRequestControl ) ) != DMR_NO_ERROR )
		{
			break;
		}
		++requestCnt;
		{
			wxCriticalSectionLocker locker(m_critSectConfig);
			++m_currentlyActiveRequests;
			if( ( m_captureQueueDepth > 0 ) && ( m_currentlyActiveRequests >= m_captureQueueDepth ) )
			{
				break;
			}
		}
	} while( true );
	if( pFramesRequested )
	{
		*pFramesRequested = requestCnt;
	}
	return requestResult;
}
